require 'spec_helper'

describe Basil::Basil do

  let (:barcodes) {
    {
      1 => 'gggg',
      2 => 'aaaa',
      3 => 'tttt',
      4 => 'cccc',
    }
  }

  let (:basil) { Basil::Basil.new barcodes }

  let (:dna) { 'ggggnnnn' }

  it 'can be created' do
    barcodes.should_not raise_error TypeError
  end

  it 'has a version number' do
    Basil::Version::STRING.should_not be_nil
  end

  it 'raises an error if duplicate barcodes are specified' do
    Basil::Basil.new({ 1 => 'gggg', 2 => 'gggg' }).should raise_error
  end

  it '#recognize returns the name of matched barcode' do
    basil.recognize(dna).first.should == 1
  end

  it 'accepts regexp for barcodes' do
    basil = Basil::Basil.new({ 1 => /^.GATC/i})
    basil.recognize("NGATC").first.should == 1
  end

  it '#recognize preserves case sensitivity when given regexp' do
    basil = Basil::Basil.new(1 => /^.GATC/)
    basil.recognize("Ngatc").should be_nil
  end

  it '#recognize is case sensitive' do
    basil.recognize(dna.upcase!).first.should == 1
  end

  it '#recognize ^ is automatically prepended to barcode regexp if strings are provided' do
    basil.recognize('aggggaaa').should be_nil
  end

  it 'trims barcode from sequence' do
    basil.recognize(dna).last.should == 'nnnn'
  end

  it 'can parse a barcodes.csv file producing a hash' do
    parsed_barcodes = File.open('spec/data/barcodes.csv') { |h| Basil::Basil.parse_barcodes h }
    parsed_barcodes.should == { 'a' => 'a'*7, 'g' => 'g'*7, 'c' => 'c'*7, 't' => 't'*7 }
  end

end
